import mapStoreys from "../assets/models/mapStoreys.js";

/**
 * @param { V2D.Viewer } viewer
 */
function addExploder(viewer, factor = 1) {
  const slider = document.getElementById("explode");

  const objectIdStoreyMap = new Map();

  viewer.scene.on("model-added", model => {
    model.objects.forEach(object => {
      const storey = mapStoreys.find(storey =>
        storey.uuids.includes(object.uuid)
      );
      if (storey) {
        objectIdStoreyMap.set(object.id, {
          storey: storey.name,
          i: mapStoreys.indexOf(storey),
        });
      }
    });
  });

  let lastMove = 0;

  slider.addEventListener("input", e => {
    const currentMove = e.currentTarget.value * factor;
    const move = currentMove - lastMove;
    lastMove = currentMove;

    function movePoint(point, amount) {
      point.x += amount;
    }

    function moveShape(shape, amount) {
      if (shape.type === "arc") {
        movePoint(shape, move * amount);
      } else {
        shape.points.forEach(point => movePoint(point, move * amount));
      }
    }

    viewer.scene.objects.forEach(object => {
      object.geometry.shapes.forEach(shape =>
        moveShape(shape, objectIdStoreyMap.get(object.id)?.i ?? 0)
      );
    });
  });
}

export default addExploder;
