/**
 * @param { V2D.Viewer } viewer
 */
function addRuller(viewer, texter, { unit = "meter", scale = 1 }, options) {
  const rulerButton = document.getElementById("ruler");
  const canvas = viewer.canvas;

  let addRulerMode = false;
  let addingRuler = false;
  let sceneObjectUpdateSubscription = null;
  let endPoint = null;

  const toggleRulerMode = () => {
    addRulerMode = !addRulerMode;
    if (!addRulerMode) {
      rulerButton.textContent = "Add ruler";
      addingRuler = false;
      canvas.style.cursor = "";
      options.enabled = true;
    } else {
      rulerButton.textContent = "Stop ruler mode";
      canvas.style.cursor = "copy";
      options.enabled = false;
    }
  };

  rulerButton.addEventListener("click", toggleRulerMode);

  window.addEventListener("keydown", e => {
    if (e.key === "r") {
      toggleRulerMode();
    }
  });

  canvas.addEventListener("click", e => {
    if (addRulerMode) {
      if (addingRuler) {
        addingRuler = false;
        addRulerMode = false;
        canvas.style.cursor = "";
        rulerButton.textContent = "Add ruler";
        viewer.scene.off(sceneObjectUpdateSubscription);
        viewer.ticker.addOnce("ruler-nextTick", () => {
          options.enabled = true;
        });
      } else {
        options.enabled = false;
        canvas.style.cursor = "crosshair";

        const { offsetX, offsetY } = e;
        const { x, y } = viewer.camera.transform.applyInverse({
          x: offsetX * window.devicePixelRatio,
          y: offsetY * window.devicePixelRatio,
        });

        const object = viewer.scene.addObject({
          lineWidth: 5,
          lineColor: 0xcccccc,
          lineOpacity: 0.5,
          pickable: false,
          geometry: [[x, y, x, y]],
        });

        const textObject = texter.addText("0", object.id, 10);

        const startPoint = object.geometry.getShape().getPoint(0);
        endPoint = object.geometry.getShape().getPoint();

        sceneObjectUpdateSubscription = viewer.scene.on(
          "object-update",
          ({ object: o, property }) => {
            if (property === "geometry" && object.id === o.id) {
              const { x: pointX, y: pointY } = object.geometry.center;

              const { x = 0, y = 0 } = viewer.camera.transform.apply({
                x: pointX,
                y: pointY,
              });

              textObject.position.x = x / window.devicePixelRatio;
              textObject.position.y = y / window.devicePixelRatio;

              const rulerLength = (
                viewer.utils.vector2D.distance(startPoint, endPoint) / scale
              ).toFixed(2);

              textObject.text = `${rulerLength} ${unit}${
                rulerLength > 1 ? "s" : ""
              }`;
            }
          }
        );

        addingRuler = true;
      }
    }
  });

  canvas.addEventListener("mousemove", e => {
    if (addingRuler) {
      const { offsetX, offsetY } = e;
      const { x, y } = viewer.camera.transform.applyInverse({
        x: offsetX * window.devicePixelRatio,
        y: offsetY * window.devicePixelRatio,
      });

      endPoint.x = x;
      endPoint.y = y;
    }
  });
}

export default addRuller;
